@@ -2,23 +2,27 @@ import copy
 import json
 import platform
 import re
+import zoneinfo
 from datetime import date, datetime, time, timedelta, timezone, tzinfo
 from decimal import Decimal
-from typing import Dict
 
 import pytest
 
-try:
-    import zoneinfo
-except ImportError:
-    # TODO: can remove this once we drop support for python 3.8
-    from backports import zoneinfo
-
 from pydantic_core import SchemaError, SchemaValidator, ValidationError, core_schema, validate_core_schema
+from pydantic_core import core_schema as cs
 
 from ..conftest import Err, PyAndJson
 
 
+@pytest.mark.parametrize(
+    'constraint',
+    ['le', 'lt', 'ge', 'gt'],
+)
+def test_constraints_schema_validation(constraint: str) -> None:
+    with pytest.raises(SchemaError, match=f"'{constraint}' must be coercible to a datetime instance"):
+        SchemaValidator(cs.datetime_schema(**{constraint: 'bad_value'}))
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -38,10 +42,13 @@ from ..conftest import Err, PyAndJson
         (Decimal('1654646400.1234568'), datetime(2022, 6, 8, 0, 0, 0, 123457, tzinfo=timezone.utc)),
         ('1654646400.1234568', datetime(2022, 6, 8, 0, 0, 0, 123457, tzinfo=timezone.utc)),
         (253_402_300_800_000, Err('should be a valid datetime, dates after 9999 are not supported as unix timestamps')),
-        (-20_000_000_000, Err('should be a valid datetime, dates before 1600 are not supported as unix timestamps')),
+        (
+            -80_000_000_000_000,
+            Err('should be a valid datetime, dates before 0000 are not supported as unix timestamps'),
+        ),
         (float('nan'), Err('Input should be a valid datetime, NaN values not permitted [type=datetime_parsing,')),
         (float('inf'), Err('Input should be a valid datetime, dates after 9999')),
-        (float('-inf'), Err('Input should be a valid datetime, dates before 1600')),
+        (float('-inf'), Err('Input should be a valid datetime, dates before 0000')),
         ('-', Err('Input should be a valid datetime or date, input is too short [type=datetime_from_date_parsing,')),
         ('+', Err('Input should be a valid datetime or date, input is too short [type=datetime_from_date_parsing,')),
         (
@@ -50,10 +57,18 @@ from ..conftest import Err, PyAndJson
                 'Input should be a valid datetime or date, day value is outside expected range [type=datetime_from_date_parsing,'
             ),
         ),
+        (
+            '0001-01-01T00:00:00.000000Z',
+            datetime(1, 1, 1, tzinfo=timezone.utc),
+        ),
+        (
+            '0000-12-31T23:59:59.999999Z',
+            Err('Input should be a valid datetime, year 0 is out of range [type=datetime_parsing,'),
+        ),
     ],
 )
 def test_datetime(input_value, expected):
-    v = SchemaValidator({'type': 'datetime'})
+    v = SchemaValidator(cs.datetime_schema())
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             result = v.validate_python(input_value)
@@ -76,7 +91,7 @@ def test_datetime(input_value, expected):
     ],
 )
 def test_datetime_strict(input_value, expected):
-    v = SchemaValidator({'type': 'datetime', 'strict': True})
+    v = SchemaValidator(cs.datetime_schema(strict=True))
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_python(input_value)
@@ -88,7 +103,7 @@ def test_datetime_strict(input_value, expected):
 def test_keep_tz():
     tz = zoneinfo.ZoneInfo('Europe/London')
     dt = datetime(2022, 6, 14, 12, 13, 14, tzinfo=tz)
-    v = SchemaValidator({'type': 'datetime'})
+    v = SchemaValidator(cs.datetime_schema())
 
     output = v.validate_python(dt)
     assert output == dt
@@ -101,7 +116,7 @@ def test_keep_tz():
 def test_keep_tz_bound():
     tz = zoneinfo.ZoneInfo('Europe/London')
     dt = datetime(2022, 6, 14, 12, 13, 14, tzinfo=tz)
-    v = SchemaValidator({'type': 'datetime', 'gt': datetime(2022, 1, 1)})
+    v = SchemaValidator(cs.datetime_schema(gt=datetime(2022, 1, 1)))
 
     output = v.validate_python(dt)
     assert output == dt
@@ -161,7 +176,7 @@ def test_datetime_json(py_and_json: PyAndJson, input_value, expected):
     ],
 )
 def test_datetime_strict_json(input_value, expected):
-    v = SchemaValidator({'type': 'datetime', 'strict': True})
+    v = SchemaValidator(cs.datetime_schema(strict=True))
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_json(json.dumps(input_value))
@@ -171,7 +186,7 @@ def test_datetime_strict_json(input_value, expected):
 
 
 def test_custom_timezone_repr():
-    output = SchemaValidator({'type': 'datetime'}).validate_python('2022-06-08T12:13:14-12:15')
+    output = SchemaValidator(cs.datetime_schema()).validate_python('2022-06-08T12:13:14-12:15')
     assert output == datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone(timedelta(hours=-12, minutes=-15)))
     assert output.tzinfo.utcoffset(output) == timedelta(hours=-12, minutes=-15)
     assert output.tzinfo.dst(output) is None
@@ -181,7 +196,7 @@ def test_custom_timezone_repr():
 
 
 def test_custom_timezone_utc_repr():
-    output = SchemaValidator({'type': 'datetime'}).validate_python('2022-06-08T12:13:14Z')
+    output = SchemaValidator(cs.datetime_schema()).validate_python('2022-06-08T12:13:14Z')
     assert output == datetime(2022, 6, 8, 12, 13, 14, tzinfo=timezone(timedelta(0)))
     assert output.tzinfo.utcoffset(output) == timedelta(0)
     assert output.tzinfo.dst(output) is None
@@ -195,19 +210,19 @@ def test_tz_comparison():
     uk_3pm = datetime(2022, 1, 1, 15, 0, 0, tzinfo=tz)
 
     # two times are the same instant, therefore le and ge are both ok
-    v = SchemaValidator({'type': 'datetime', 'le': uk_3pm}).validate_python('2022-01-01T16:00:00+01:00')
+    v = SchemaValidator(cs.datetime_schema(le=uk_3pm)).validate_python('2022-01-01T16:00:00+01:00')
     assert v == datetime(2022, 1, 1, 16, 0, 0, tzinfo=timezone(timedelta(hours=1)))
 
-    v = SchemaValidator({'type': 'datetime', 'ge': uk_3pm}).validate_python('2022-01-01T16:00:00+01:00')
+    v = SchemaValidator(cs.datetime_schema(ge=uk_3pm)).validate_python('2022-01-01T16:00:00+01:00')
     assert v == datetime(2022, 1, 1, 16, 0, 0, tzinfo=timezone(timedelta(hours=1)))
 
     # but not gt
     with pytest.raises(ValidationError, match=r'Input should be greater than 2022-01-01T15:00:00Z \[type=greater_than'):
-        SchemaValidator({'type': 'datetime', 'gt': uk_3pm}).validate_python('2022-01-01T16:00:00+01:00')
+        SchemaValidator(cs.datetime_schema(gt=uk_3pm)).validate_python('2022-01-01T16:00:00+01:00')
 
 
 def test_tz_info_deepcopy():
-    output = SchemaValidator({'type': 'datetime'}).validate_python('2023-02-15T16:23:44.037Z')
+    output = SchemaValidator(cs.datetime_schema()).validate_python('2023-02-15T16:23:44.037Z')
     c = copy.deepcopy(output)
     assert repr(output.tzinfo) == 'TzInfo(UTC)'
     assert repr(c.tzinfo) == 'TzInfo(UTC)'
@@ -215,7 +230,7 @@ def test_tz_info_deepcopy():
 
 
 def test_tz_info_copy():
-    output = SchemaValidator({'type': 'datetime'}).validate_python('2023-02-15T16:23:44.037Z')
+    output = SchemaValidator(cs.datetime_schema()).validate_python('2023-02-15T16:23:44.037Z')
     c = copy.copy(output)
     assert repr(output.tzinfo) == 'TzInfo(UTC)'
     assert repr(c.tzinfo) == 'TzInfo(UTC)'
@@ -233,7 +248,7 @@ def test_custom_tz():
         def tzname(self, _dt):
             return 'CustomTZ'
 
-    schema = SchemaValidator({'type': 'datetime', 'gt': datetime(2022, 1, 1, 15, 0, 0)})
+    schema = SchemaValidator(cs.datetime_schema(gt=datetime(2022, 1, 1, 15, 0, 0)))
 
     dt = datetime(2022, 1, 1, 16, 0, 0, tzinfo=CustomTz())
     outcome = schema.validate_python(dt)
@@ -247,7 +262,7 @@ def test_custom_invalid_tz():
         def tzname(self, _dt):
             return 'CustomTZ'
 
-    schema = SchemaValidator({'type': 'datetime', 'gt': datetime(2022, 1, 1, 15, 0, 0)})
+    schema = SchemaValidator(cs.datetime_schema(gt=datetime(2022, 1, 1, 15, 0, 0)))
 
     dt = datetime(2022, 1, 1, 16, 0, 0, tzinfo=CustomTz())
     # perhaps this should be a ValidationError? but we don't catch other errors
@@ -272,7 +287,7 @@ def test_custom_invalid_tz():
 
 
 def test_dict_py():
-    v = SchemaValidator({'type': 'dict', 'keys_schema': {'type': 'datetime'}, 'values_schema': {'type': 'int'}})
+    v = SchemaValidator(cs.dict_schema(keys_schema=cs.datetime_schema(), values_schema=cs.int_schema()))
     assert v.validate_python({datetime(2000, 1, 1): 2, datetime(2000, 1, 2): 4}) == {
         datetime(2000, 1, 1): 2,
         datetime(2000, 1, 2): 4,
@@ -288,11 +303,11 @@ def test_dict(py_and_json: PyAndJson):
 
 
 def test_union():
-    v = SchemaValidator({'type': 'union', 'choices': [{'type': 'str'}, {'type': 'datetime'}]})
+    v = SchemaValidator(cs.union_schema(choices=[cs.str_schema(), cs.datetime_schema()]))
     assert v.validate_python('2022-01-02T00:00') == '2022-01-02T00:00'
     assert v.validate_python(datetime(2022, 1, 2)) == datetime(2022, 1, 2)
 
-    v = SchemaValidator({'type': 'union', 'choices': [{'type': 'datetime'}, {'type': 'str'}]})
+    v = SchemaValidator(cs.union_schema(choices=[cs.datetime_schema(), cs.str_schema()]))
     assert v.validate_python('2022-01-02T00:00') == '2022-01-02T00:00'
     assert v.validate_python(datetime(2022, 1, 2)) == datetime(2022, 1, 2)
 
@@ -497,7 +512,7 @@ def test_tz_constraint_wrong():
 
 def test_tz_hash() -> None:
     v = SchemaValidator(core_schema.datetime_schema())
-    lookup: Dict[datetime, str] = {}
+    lookup: dict[datetime, str] = {}
     for day in range(1, 10):
         input_str = f'2022-06-{day:02}T12:13:14-12:15'
         validated = v.validate_python(input_str)
