@@ -9,7 +9,8 @@ from typing import Any
 import pytest
 from dirty_equals import FunctionCheck, IsStr
 
-from pydantic_core import SchemaValidator, ValidationError
+from pydantic_core import SchemaError, SchemaValidator, ValidationError
+from pydantic_core import core_schema as cs
 
 from ..conftest import Err, PyAndJson, plain_repr
 
@@ -18,6 +19,17 @@ class DecimalSubclass(Decimal):
     pass
 
 
+# Note: there's another constraint validation (allow_inf_nan=True cannot be used with max_digits or decimal_places).
+# but it is tested in Pydantic:
+@pytest.mark.parametrize(
+    'constraint',
+    ['multiple_of', 'le', 'lt', 'ge', 'gt'],
+)
+def test_constraints_schema_validation(constraint: str) -> None:
+    with pytest.raises(SchemaError, match=f"'{constraint}' must be coercible to a Decimal instance"):
+        SchemaValidator(cs.decimal_schema(**{constraint: 'bad_value'}))
+
+
 @pytest.mark.parametrize(
     'input_value,expected',
     [
@@ -90,7 +102,7 @@ def test_decimal(py_and_json: PyAndJson, input_value, expected):
     ids=repr,
 )
 def test_decimal_strict_py(input_value, expected):
-    v = SchemaValidator({'type': 'decimal', 'strict': True})
+    v = SchemaValidator(cs.decimal_schema(strict=True))
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_python(input_value)
@@ -120,7 +132,7 @@ def test_decimal_strict_py(input_value, expected):
     ids=repr,
 )
 def test_decimal_strict_json(input_value, expected):
-    v = SchemaValidator({'type': 'decimal', 'strict': True})
+    v = SchemaValidator(cs.decimal_schema(strict=True))
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_json(json.dumps(input_value))
@@ -171,24 +183,44 @@ def test_decimal_kwargs(py_and_json: PyAndJson, kwargs: dict[str, Any], input_va
 @pytest.mark.parametrize(
     'multiple_of,input_value,error',
     [
-        (0.5, 0.5, None),
-        (0.5, 1, None),
+        # Test cases for multiples of 0.5
+        *[(0.5, round(i * 0.5, 1), None) for i in range(-4, 5)],
+        (0.5, 0.49, Err('Input should be a multiple of 0.5')),
         (0.5, 0.6, Err('Input should be a multiple of 0.5')),
-        (0.5, 0.51, Err('Input should be a multiple of 0.5')),
+        (0.5, -0.75, Err('Input should be a multiple of 0.5')),
         (0.5, 0.501, Err('Input should be a multiple of 0.5')),
         (0.5, 1_000_000.5, None),
         (0.5, 1_000_000.49, Err('Input should be a multiple of 0.5')),
+        (0.5, int(5e10), None),
+        # Test cases for multiples of 0.1
+        *[(0.1, round(i * 0.1, 1), None) for i in range(-10, 11)],
         (0.1, 0, None),
-        (0.1, 0.0, None),
-        (0.1, 0.2, None),
-        (0.1, 0.3, None),
-        (0.1, 0.4, None),
-        (0.1, 0.5, None),
         (0.1, 0.5001, Err('Input should be a multiple of 0.1')),
+        (0.1, 0.05, Err('Input should be a multiple of 0.1')),
+        (0.1, -0.15, Err('Input should be a multiple of 0.1')),
+        (0.1, 1_000_000.1, None),
+        (0.1, 1_000_000.05, Err('Input should be a multiple of 0.1')),
         (0.1, 1, None),
-        (0.1, 1.0, None),
         (0.1, int(5e10), None),
-        (2.0, -2.0, None),
+        # Test cases for multiples of 2.0
+        *[(2.0, i * 2.0, None) for i in range(-5, 6)],
+        (2.0, -2.1, Err('Input should be a multiple of 2')),
+        (2.0, -3.0, Err('Input should be a multiple of 2')),
+        (2.0, 1_000_002.0, None),
+        (2.0, 1_000_001.0, Err('Input should be a multiple of 2')),
+        (2.0, int(5e10), None),
+        # Test cases for multiples of 0.01
+        *[(0.01, round(i * 0.01, 2), None) for i in range(-10, 11)],
+        (0.01, 0.005, Err('Input should be a multiple of 0.01')),
+        (0.01, -0.015, Err('Input should be a multiple of 0.01')),
+        (0.01, 1_000_000.01, None),
+        (0.01, 1_000_000.005, Err('Input should be a multiple of 0.01')),
+        (0.01, int(5e10), None),
+        # Test cases for values very close to zero
+        (0.1, 0.00001, Err('Input should be a multiple of 0.1')),
+        (0.1, -0.00001, Err('Input should be a multiple of 0.1')),
+        (0.01, 0.00001, Err('Input should be a multiple of 0.01')),
+        (0.01, -0.00001, Err('Input should be a multiple of 0.01')),
     ],
     ids=repr,
 )
@@ -204,9 +236,7 @@ def test_decimal_multiple_of(py_and_json: PyAndJson, multiple_of: float, input_v
 
 
 def test_union_decimal_py():
-    v = SchemaValidator(
-        {'type': 'union', 'choices': [{'type': 'decimal', 'strict': True}, {'type': 'decimal', 'multiple_of': 7}]}
-    )
+    v = SchemaValidator(cs.union_schema(choices=[cs.decimal_schema(strict=True), cs.decimal_schema(multiple_of=7)]))
     assert v.validate_python('14') == 14
     assert v.validate_python(Decimal(5)) == 5
     with pytest.raises(ValidationError) as exc_info:
@@ -230,9 +260,7 @@ def test_union_decimal_py():
 
 
 def test_union_decimal_json():
-    v = SchemaValidator(
-        {'type': 'union', 'choices': [{'type': 'decimal', 'strict': True}, {'type': 'decimal', 'multiple_of': 7}]}
-    )
+    v = SchemaValidator(cs.union_schema(choices=[cs.decimal_schema(strict=True), cs.decimal_schema(multiple_of=7)]))
     assert v.validate_json(json.dumps('14')) == 14
     assert v.validate_json(json.dumps('5')) == 5
 
@@ -255,21 +283,21 @@ def test_union_decimal_simple(py_and_json: PyAndJson):
 
 
 def test_decimal_repr():
-    v = SchemaValidator({'type': 'decimal'})
+    v = SchemaValidator(cs.decimal_schema())
     assert plain_repr(v).startswith(
         'SchemaValidator(title="decimal",validator=Decimal(DecimalValidator{strict:false,allow_inf_nan:false'
     )
-    v = SchemaValidator({'type': 'decimal', 'strict': True})
+    v = SchemaValidator(cs.decimal_schema(strict=True))
     assert plain_repr(v).startswith(
         'SchemaValidator(title="decimal",validator=Decimal(DecimalValidator{strict:true,allow_inf_nan:false'
     )
-    v = SchemaValidator({'type': 'decimal', 'multiple_of': 7})
+    v = SchemaValidator(cs.decimal_schema(multiple_of=7))
     assert plain_repr(v).startswith('SchemaValidator(title="decimal",validator=Decimal(')
 
 
 @pytest.mark.parametrize('input_value,expected', [(Decimal('1.23'), Decimal('1.23')), (Decimal('1'), Decimal('1.0'))])
 def test_decimal_not_json(input_value, expected):
-    v = SchemaValidator({'type': 'decimal'})
+    v = SchemaValidator(cs.decimal_schema())
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_python(input_value)
@@ -363,7 +391,7 @@ def test_non_finite_json_values(py_and_json: PyAndJson, input_value, allow_inf_n
     ],
 )
 def test_non_finite_decimal_values(strict, input_value, allow_inf_nan, expected):
-    v = SchemaValidator({'type': 'decimal', 'allow_inf_nan': allow_inf_nan, 'strict': strict})
+    v = SchemaValidator(cs.decimal_schema(allow_inf_nan=allow_inf_nan, strict=strict))
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_python(input_value)
@@ -399,7 +427,7 @@ def test_non_finite_decimal_values(strict, input_value, allow_inf_nan, expected)
     ],
 )
 def test_non_finite_constrained_decimal_values(input_value, allow_inf_nan, expected):
-    v = SchemaValidator({'type': 'decimal', 'allow_inf_nan': allow_inf_nan, 'gt': 0})
+    v = SchemaValidator(cs.decimal_schema(allow_inf_nan=allow_inf_nan, gt=0))
     if isinstance(expected, Err):
         with pytest.raises(ValidationError, match=re.escape(expected.message)):
             v.validate_python(input_value)
@@ -437,12 +465,12 @@ def test_non_finite_constrained_decimal_values(input_value, allow_inf_nan, expec
     ],
 )
 def test_validate_scientific_notation_from_json(input_value, expected):
-    v = SchemaValidator({'type': 'decimal'})
+    v = SchemaValidator(cs.decimal_schema())
     assert v.validate_json(input_value) == expected
 
 
 def test_validate_max_digits_and_decimal_places() -> None:
-    v = SchemaValidator({'type': 'decimal', 'max_digits': 5, 'decimal_places': 2})
+    v = SchemaValidator(cs.decimal_schema(max_digits=5, decimal_places=2))
 
     # valid inputs
     assert v.validate_json('1.23') == Decimal('1.23')
@@ -461,9 +489,29 @@ def test_validate_max_digits_and_decimal_places() -> None:
 
 
 def test_validate_max_digits_and_decimal_places_edge_case() -> None:
-    v = SchemaValidator({'type': 'decimal', 'max_digits': 34, 'decimal_places': 18})
+    v = SchemaValidator(cs.decimal_schema(max_digits=34, decimal_places=18))
 
     # valid inputs
     assert v.validate_python(Decimal('9999999999999999.999999999999999999')) == Decimal(
         '9999999999999999.999999999999999999'
     )
+
+
+def test_str_validation_w_strict() -> None:
+    s = SchemaValidator(cs.decimal_schema(strict=True))
+
+    with pytest.raises(ValidationError):
+        assert s.validate_python('1.23')
+
+
+def test_str_validation_w_lax() -> None:
+    s = SchemaValidator(cs.decimal_schema(strict=False))
+
+    assert s.validate_python('1.23') == Decimal('1.23')
+
+
+def test_union_with_str_prefers_str() -> None:
+    s = SchemaValidator(cs.union_schema([cs.decimal_schema(), cs.str_schema()]))
+
+    assert s.validate_python('1.23') == '1.23'
+    assert s.validate_python(1.23) == Decimal('1.23')
